@@ -27,6 +27,7 @@ from oslo_utils import uuidutils
 from ironic.common import driver_factory
 from ironic.common import exception
 from ironic.common import states
+from ironic.common import utils as common_utils
 from ironic.conductor import base_manager
 from ironic.conductor import manager
 from ironic.conductor import notification_utils
@@ -193,11 +194,11 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertTrue(mock_df.called)
             self.assertFalse(mock_reg.called)
 
-    def test_start_fails_on_no_enabled_interfaces(self):
-        self.config(enabled_boot_interfaces=[])
-        self.assertRaisesRegex(exception.ConfigInvalid,
-                               'options enabled_boot_interfaces',
-                               self.service.init_host)
+    def test_start_with_no_enabled_interfaces(self):
+        self.config(enabled_boot_interfaces=[],
+                    enabled_deploy_interfaces=[],
+                    enabled_hardware_types=['fake-hardware'])
+        self._start_service()
 
     @mock.patch.object(base_manager, 'LOG', autospec=True)
     @mock.patch.object(driver_factory, 'HardwareTypesFactory', autospec=True)
@@ -310,16 +311,32 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         # 3 without reuse of the database connection.
         self.assertEqual(2, mock_dbapi.call_count)
 
+    def test_start_with_json_rpc(self):
+        CONF.set_override('rpc_transport', 'json-rpc')
+        CONF.set_override('host', 'foo.bar.baz')
+        self._start_service()
+        res = objects.Conductor.get_by_hostname(self.context, self.hostname)
+        self.assertEqual(self.hostname, res['hostname'])
 
-class CheckInterfacesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
-    def test__check_enabled_interfaces_success(self):
-        base_manager._check_enabled_interfaces()
+    def test_start_with_json_rpc_port(self):
+        CONF.set_override('rpc_transport', 'json-rpc')
+        CONF.set_override('host', 'foo.bar.baz')
+        CONF.set_override('port', 8192, group='json_rpc')
 
-    def test__check_enabled_interfaces_failure(self):
-        self.config(enabled_boot_interfaces=[])
-        self.assertRaisesRegex(exception.ConfigInvalid,
-                               'options enabled_boot_interfaces',
-                               base_manager._check_enabled_interfaces)
+        self._start_service()
+        res = objects.Conductor.get_by_hostname(self.context,
+                                                self.service.host)
+        self.assertEqual(f'{self.hostname}:8192', res['hostname'])
+
+    def test_start_without_jsonrpc_port_pined_version(self):
+        CONF.set_override('rpc_transport', 'json-rpc')
+        CONF.set_override('host', 'foo.bar.baz')
+        CONF.set_override('port', 8192, group='json_rpc')
+        CONF.set_override('pin_release_version', '21.4')
+        self._start_service()
+        res = objects.Conductor.get_by_hostname(self.context,
+                                                self.service.host)
+        self.assertEqual(self.hostname, res['hostname'])
 
 
 class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
@@ -333,12 +350,24 @@ class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                                    'is_set', autospec=True) as mock_is_set:
                 mock_is_set.side_effect = [False, True]
                 self.service._conductor_service_record_keepalive()
-            mock_touch.assert_called_once_with(self.hostname)
+            mock_touch.assert_not_called()
+            with mock.patch.object(self.service._keepalive_evt,
+                                   'is_set', autospec=True) as mock_is_set:
+                mock_is_set.side_effect = [False, True]
+                with mock.patch.object(common_utils, 'is_ironic_using_sqlite',
+                                       autospec=True) as mock_is_sqlite:
+                    mock_is_sqlite.return_value = False
+                    self.service._conductor_service_record_keepalive()
+                    self.assertEqual(1, mock_is_sqlite.call_count)
+            mock_touch.assert_called_once_with(self.hostname, online=True)
 
-    def test__conductor_service_record_keepalive_failed_db_conn(self):
+    @mock.patch.object(common_utils, 'is_ironic_using_sqlite', autospec=True)
+    def test__conductor_service_record_keepalive_failed_db_conn(
+            self, is_sqlite_mock):
         self._start_service()
         # avoid wasting time at the event.wait()
         CONF.set_override('heartbeat_interval', 0, 'conductor')
+        is_sqlite_mock.return_value = False
         with mock.patch.object(self.dbapi, 'touch_conductor',
                                autospec=True) as mock_touch:
             mock_touch.side_effect = [None, db_exception.DBConnectionError(),
@@ -348,11 +377,15 @@ class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                 mock_is_set.side_effect = [False, False, False, True]
                 self.service._conductor_service_record_keepalive()
             self.assertEqual(3, mock_touch.call_count)
+        self.assertEqual(1, is_sqlite_mock.call_count)
 
-    def test__conductor_service_record_keepalive_failed_error(self):
+    @mock.patch.object(common_utils, 'is_ironic_using_sqlite', autospec=True)
+    def test__conductor_service_record_keepalive_failed_error(self,
+                                                              is_sqlite_mock):
         self._start_service()
-        # avoid wasting time at the event.wait()
+        # minimal time at the event.wait()
         CONF.set_override('heartbeat_interval', 0, 'conductor')
+        is_sqlite_mock.return_value = False
         with mock.patch.object(self.dbapi, 'touch_conductor',
                                autospec=True) as mock_touch:
             mock_touch.side_effect = [None, Exception(),
@@ -362,6 +395,7 @@ class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                 mock_is_set.side_effect = [False, False, False, True]
                 self.service._conductor_service_record_keepalive()
             self.assertEqual(3, mock_touch.call_count)
+        self.assertEqual(1, is_sqlite_mock.call_count)
 
 
 class ManagerSpawnWorkerTestCase(tests_base.TestCase):
@@ -505,9 +539,11 @@ class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         obj_utils.create_test_node(
             self.context,
             uuid=uuidutils.generate_uuid(),
-            driver='fake-hardware'
+            driver='fake-hardware',
         )
-        self._start_service()
+        # Enable consoles *after* service has started, otherwise it races
+        # as the service startup also launches consoles.
+        self._start_service(start_consoles=False)
         self.service._start_consoles(self.context)
         self.assertEqual(2, mock_start_console.call_count)
         mock_notify.assert_has_calls(
@@ -542,6 +578,12 @@ class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                        fields.NotificationStatus.START),
              mock.call(mock.ANY, 'console_restore',
                        fields.NotificationStatus.ERROR)])
+        history = objects.NodeHistory.list_by_node_id(self.context,
+                                                      test_node.id)
+        entry = history[0]
+        self.assertEqual('startup failure', entry['event_type'])
+        self.assertEqual('ERROR', entry['severity'])
+        self.assertIsNotNone(entry['event'])
 
     @mock.patch.object(base_manager, 'LOG', autospec=True)
     def test__start_consoles_node_locked(self, log_mock, mock_notify,
@@ -599,3 +641,9 @@ class MiscTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.service._fail_transient_state(states.DEPLOYING, 'unknown err')
         node.refresh()
         self.assertEqual(states.DEPLOYFAIL, node.provision_state)
+        history = objects.NodeHistory.list_by_node_id(self.context,
+                                                      node.id)
+        entry = history[0]
+        self.assertEqual('transition', entry['event_type'])
+        self.assertEqual('ERROR', entry['severity'])
+        self.assertEqual('unknown err', entry['event'])
